#include "context.h"
#include "sequence.h"

const char *feature_type_names[] = { "tag", "seg" };

////////////////////////////////////////////////////////////

PtContext::PtContext(const Domain &D, Sequence &seq, bool use_averaged) :
  Context(D, seq, use_averaged), i(-1), prev2_tag(-1), prev_tag(-1), curr_tag(-1) { }

void PtContext::output(ostream &out) const {
  out << "i=" << i << '/' << len(seq)
      << ", prev2_tag=" << D.tag_db[prev2_tag]
      << ", prev_tag=" << D.tag_db[prev_tag]
      << ", curr_tag=" << D.tag_db[curr_tag];
}

void PtContext::step(int new_tag) {
  i++;
  prev2_tag = prev_tag;
  prev_tag = curr_tag;
  curr_tag = new_tag;
}

////////////////////////////////////////////////////////////

SegContext::SegContext(const Domain &D, Sequence &seq, bool use_averaged) :
  Context(D, seq, use_averaged), i(-1), n(1), prev_seg(-1), curr_seg(-1) { }

void SegContext::output(ostream &out) const {
  out << "i=" << i << '/' << len(seq) << ", n=" << n
      << ", prev_seg=" << D.seg_db[prev_seg]
      << ", curr_seg=" << D.seg_db[curr_seg];
}

void SegContext::step(const Seg &new_seg) {
  i += n;
  n = new_seg.n;
  prev_seg = curr_seg;
  curr_seg = new_seg.a;
}

////////////////////////////////////////////////////////////

ostream &operator<<(ostream &out, const Context &c) {
  c.output(out);
  return out;
}
