#ifndef __CONTEXT_H__
#define __CONTEXT_H__

#include "domain.h"

struct Sequence;
struct FeatureCache;

enum FeatureType { FTYPE_PT, FTYPE_SEG };

extern const char *feature_type_names[];

////////////////////////////////////////////////////////////

struct Context {
  Context(const Domain &D, Sequence &seq, bool use_averaged) :
    D(D), seq(seq), use_averaged(use_averaged) { }

  virtual void output(ostream &out) const = 0;

  const Domain &D;
  Sequence &seq;
  bool use_averaged;
};

ostream &operator<<(ostream &out, const Context &c);

////////////////////////////////////////////////////////////

struct PtContext : public Context {
  PtContext(const Domain &D, Sequence &seq, bool use_averaged);

  void output(ostream &out) const;
  void step(int new_tag);

  int i; // Position of the point.
  int prev2_tag, prev_tag, curr_tag;
};

struct SegContext : public Context {
  SegContext(const Domain &D, Sequence &seq, bool use_averaged);

  void output(ostream &out) const;
  void step(const Seg &new_seg);

  int i, n; // Starting point of segment and length of segment
  int prev_seg, curr_seg;
};

#endif
