#include "domain.h"
#include "token.h"
#include "basic/stl-utils.h"

////////////////////////////////////////////////////////////

Domain::Domain(const string &tag_domain_str) : boundary_token(NULL), no_seg(-1), no_seg_tag(-1) {
  if(tag_domain_str == "pos")        tag_domain = POS;
  else if(tag_domain_str == "chunk") tag_domain = CHUNK;
  else if(tag_domain_str == "ent")   tag_domain = ENT;
  else assert2(false, "Unknown domain: " << tag_domain_str);

#if 1
  // Need to initialize after domain has been set.
  if(is_seg()) {
    no_seg_tag = get_tag(SegTag::no_seg_str);
    no_seg = get_seg(SegTag::no_seg_str);
  }
#endif
}

Domain::~Domain() {
  if(boundary_token) delete boundary_token;
}

// Modifies: tag_db, seg_db, stags
int Domain::get_tag(const char *s) {
  if(is_seg()) get_seg(s);
  return tag_db[s];
}

int Domain::get_seg(const char *s) {
  assert(is_seg());
  SegTag &stag = expand_get(stags, tag_db[s], SegTag());
  stag.parse(s, seg_db);
  return stag.a;
}

void Domain::read(const char *file) {
  track("Domain::read()", file, true);
  ifstream in(file); assert(in);
  int n, N;

  in >> N; n =   word_db.read(in, N, false); assert(n == N);
  in >> N; n =    tag_db.read(in, N, false); assert(n == N);
  in >> N; n =    seg_db.read(in, N, false); assert(n == N);
  in >> N; n =    pos_db.read(in, N, false); assert(n == N);
  in >> N; n =  chunk_db.read(in, N, false); assert(n == N);
  in >> N; n =    ent_db.read(in, N, false); assert(n == N);
  in >> N; n = phrase_db.read(in, N, false); assert(n == N);

  in >> N;
  stags.resize(N);
  foridx(i, N) in >> stags[i];
}

void Domain::write(const char *file) {
  track("Domain::write()", file, true);
  ofstream out(file); assert(out);

  out <<   word_db.size() << endl;   word_db.write(out); out << endl;
  out <<    tag_db.size() << endl;    tag_db.write(out); out << endl;
  out <<    seg_db.size() << endl;    seg_db.write(out); out << endl;
  out <<    pos_db.size() << endl;    pos_db.write(out); out << endl;
  out <<  chunk_db.size() << endl;  chunk_db.write(out); out << endl;
  out <<    ent_db.size() << endl;    ent_db.write(out); out << endl;
  out << phrase_db.size() << endl; phrase_db.write(out); out << endl;
  // TODO: handle spaces

  out << len(stags) << endl;
  forvec(_, const SegTag &, stag, stags)
    out << stag << endl;
}

int Domain::stag_to_tag(SegTag stag) const {
  // TODO: right now, this is naively slow
  int tag = index_of(stags, stag);
  assert(tag != -1);
  return tag;
}
