#ifndef __EXAMPLE_SET_H__
#define __EXAMPLE_SET_H__

#include "feature-set.h"
#include "performance.h"

////////////////////////////////////////////////////////////

extern int boundary_size;

class ExampleSet {
public:
  ExampleSet(const string &tag_domain_str);

  void read(const char *train_file, const char *test_file, const char *test2_file, double train_frac);
  void read(const char *file, SequenceVec &sequences, double keep_frac);
  void read(istream &in, SequenceVec &sequences, double keep_frac);

  bool seq_has_content(const Sequence *seq) const { return seq->size() > boundary_size; }
  void add_boundary(Sequence *seq);
  Sequence *new_seq(SequenceVec &sequences);
  Sequence *flush_seq(Sequence *seq, SequenceVec &sequences);
  void clear_seq(Sequence *seq);

  Performance process_sequences(FeatureType type, const char *name, SequenceVec &sequences,
    bool use_averaged, bool fix_weights, bool output_eval, const BoolVec &use, const IntVec &perm);

  void finish_initialization();
  void precompute_features();
  void precompute_features(SequenceVec &sequences);

  // Training
  void train_perceptron();
  void train_perceptron_with_active_learning();

  int pick_examples(BoolVec &in_set, int n, bool pick_randomly);

  void steal_train_data_for_cross_validation(double ratio);

  static int compute_max_seg_len(const SequenceVec &sequences);

  static int num_words(const SequenceVec &sequences, const BoolVec &use);

public:
  Domain D;
  FeatureSet features;

  SequenceVec train_sequences, validation_sequences, test_sequences, test2_sequences;

  ofstream eval_out;
};

#endif
