#include "feature-cache.h"

const IntDouble invalid_fv = IntDouble(-1, 0);
const int invalid_f = -1;
const IntDoubleVec invalid_fvs = IntDoubleVec(1, invalid_fv);
const IntVec invalid_fs = IntVec(1, invalid_f);

////////////////////////////////////////////////////////////
// Multi

bool MultiPtFeatureCache::get(const Context &context, int ft_id, IntDoubleVec &feature_values) {
  const PtContext &c = (const PtContext &)context;
  feature_values = expand_get(i2fvs, c.i, ft_id, invalid_fvs);
  if(len(feature_values) == 1 && feature_values[0].first == invalid_f) {
    feature_values.clear();
    return false;
  }
  return true;
}

void MultiPtFeatureCache::raw_get(const Context &context, int ft_id, IntDoubleVec &feature_values) const {
  const PtContext &c = (const PtContext &)context;
  feature_values = i2fvs[c.i][ft_id];
}

void MultiPtFeatureCache::put(const Context &context, int ft_id, const IntDoubleVec &feature_values) {
  const PtContext &c = (const PtContext &)context;
  expand_get(i2fvs, c.i, ft_id, invalid_fvs) = feature_values;
}

bool MultiSegFeatureCache::get(const Context &context, int ft_id, IntDoubleVec &feature_values) {
  const SegContext &c = (const SegContext &)context;
  feature_values = expand_get(in2fvs, c.i, c.n, ft_id, invalid_fvs);
  if(len(feature_values) == 1 && feature_values[0].first == invalid_f) {
    feature_values.clear();
    return false;
  }
  return true;
}

void MultiSegFeatureCache::raw_get(const Context &context, int ft_id, IntDoubleVec &feature_values) const {
  const SegContext &c = (const SegContext &)context;
  feature_values = in2fvs[c.i][c.n][ft_id];
}

void MultiSegFeatureCache::put(const Context &context, int ft_id, const IntDoubleVec &feature_values) {
  const SegContext &c = (const SegContext &)context;
  expand_get(in2fvs, c.i, c.n, ft_id, invalid_fvs) = feature_values;
}

////////////////////////////////////////////////////////////
// MultiBool

bool MultiBoolPtFeatureCache::get(const Context &context, int ft_id, IntVec &features) {
  const PtContext &c = (const PtContext &)context;
  features = expand_get(i2fs, c.i, ft_id, invalid_fs);
  if(len(features) == 1 && features[0] == invalid_f) {
    features.clear();
    return false;
  }
  return true;
}

void MultiBoolPtFeatureCache::raw_get(const Context &context, int ft_id, IntVec &features) const {
  const PtContext &c = (const PtContext &)context;
  features = i2fs[c.i][ft_id];
}

void MultiBoolPtFeatureCache::put(const Context &context, int ft_id, const IntVec &features) {
  const PtContext &c = (const PtContext &)context;
  expand_get(i2fs, c.i, ft_id, invalid_fs) = features;
}

bool MultiBoolSegFeatureCache::get(const Context &context, int ft_id, IntVec &features) {
  const SegContext &c = (const SegContext &)context;
  features = expand_get(in2fs, c.i, c.n, ft_id, invalid_fs);
  if(len(features) == 1 && features[0] == invalid_f) {
    features.clear();
    return false;
  }
  return true;
}

void MultiBoolSegFeatureCache::raw_get(const Context &context, int ft_id, IntVec &features) const {
  const SegContext &c = (const SegContext &)context;
  features = in2fs[c.i][c.n][ft_id];
}

void MultiBoolSegFeatureCache::put(const Context &context, int ft_id, const IntVec &features) {
  const SegContext &c = (const SegContext &)context;
  expand_get(in2fs, c.i, c.n, ft_id, invalid_fs) = features;
}

////////////////////////////////////////////////////////////
// Single

IntDouble SinglePtFeatureCache::get(const Context &context, int ft_id) {
  const PtContext &c = (const PtContext &)context;
  return expand_get(i2fv, c.i, ft_id, invalid_fv);
}

IntDouble SinglePtFeatureCache::raw_get(const Context &context, int ft_id) const {
  const PtContext &c = (const PtContext &)context;
  return i2fv[c.i][ft_id];
}

void SinglePtFeatureCache::put(const Context &context, int ft_id, IntDouble fv) {
  const PtContext &c = (const PtContext &)context;
  expand_get(i2fv, c.i, ft_id, invalid_fv) = fv;
}

IntDouble SingleSegFeatureCache::get(const Context &context, int ft_id) {
  const SegContext &c = (const SegContext &)context;
  return expand_get(in2fv, c.i, c.n, ft_id, invalid_fv);
}

IntDouble SingleSegFeatureCache::raw_get(const Context &context, int ft_id) const {
  const SegContext &c = (const SegContext &)context;
  return in2fv[c.i][c.n][ft_id];
}

void SingleSegFeatureCache::put(const Context &context, int ft_id, IntDouble fv) {
  const SegContext &c = (const SegContext &)context;
  expand_get(in2fv, c.i, c.n, ft_id, invalid_fv) = fv;
}

////////////////////////////////////////////////////////////
// SingleBool

int SingleBoolPtFeatureCache::get(const Context &context, int ft_id) {
  const PtContext &c = (const PtContext &)context;
  return expand_get(i2f, c.i, ft_id, invalid_f);
}

int SingleBoolPtFeatureCache::raw_get(const Context &context, int ft_id) const {
  const PtContext &c = (const PtContext &)context;
  return i2f[c.i][ft_id];
}

void SingleBoolPtFeatureCache::put(const Context &context, int ft_id, int f) {
  const PtContext &c = (const PtContext &)context;
  expand_get(i2f, c.i, ft_id, invalid_f) = f;
}

int SingleBoolSegFeatureCache::get(const Context &context, int ft_id) {
  const SegContext &c = (const SegContext &)context;
  return expand_get(in2f, c.i, c.n, ft_id, invalid_f);
}

int SingleBoolSegFeatureCache::raw_get(const Context &context, int ft_id) const {
  const SegContext &c = (const SegContext &)context;
  return in2f[c.i][c.n][ft_id];
}

void SingleBoolSegFeatureCache::put(const Context &context, int ft_id, int f) {
  const SegContext &c = (const SegContext &)context;
  expand_get(in2f, c.i, c.n, ft_id, invalid_f) = f;
}
