#ifndef __FEATURE_CACHE_H__
#define __FEATURE_CACHE_H__

#include "context.h"

// Feature caches only depend on the input sequence, not the tags.

////////////////////////////////////////////////////////////
// Multi

struct MultiFeatureCache {
  virtual ~MultiFeatureCache() { }
  virtual bool get(const Context &context, int ft_id, IntDoubleVec &feature_values) = 0;
  virtual void put(const Context &context, int ft_id, const IntDoubleVec &feature_values) = 0;
  virtual void raw_get(const Context &context, int ft_id, IntDoubleVec &feature_values) const = 0;
};

struct MultiPtFeatureCache : public MultiFeatureCache {
  bool get(const Context &context, int ft_id, IntDoubleVec &feature_values);
  void put(const Context &context, int ft_id, const IntDoubleVec &feature_values);
  void raw_get(const Context &context, int ft_id, IntDoubleVec &feature_values) const;
  IntDoubleVecVecVec i2fvs; // index in the sequence -> feature template ID -> [f, v]
};

struct MultiSegFeatureCache : public MultiFeatureCache {
  bool get(const Context &context, int ft_id, IntDoubleVec &feature_values);
  void put(const Context &context, int ft_id, const IntDoubleVec &feature_values);
  void raw_get(const Context &context, int ft_id, IntDoubleVec &feature_values) const;
  IntDoubleVecVecVecVec in2fvs; // index in the sequence, length of segment -> feature template ID -> [f, v]
};

////////////////////////////////////////////////////////////
// MultiBool

struct MultiBoolFeatureCache {
  virtual ~MultiBoolFeatureCache() { }
  virtual bool get(const Context &context, int ft_id, IntVec &features) = 0;
  virtual void put(const Context &context, int ft_id, const IntVec &features) = 0;
  virtual void raw_get(const Context &context, int ft_id, IntVec &features) const = 0;
};

struct MultiBoolPtFeatureCache : public MultiBoolFeatureCache {
  bool get(const Context &context, int ft_id, IntVec &features);
  void put(const Context &context, int ft_id, const IntVec &features);
  void raw_get(const Context &context, int ft_id, IntVec &features) const;
  IntVecVecVec i2fs; // index in the sequence -> feature template ID -> [f]
};

struct MultiBoolSegFeatureCache : public MultiBoolFeatureCache {
  bool get(const Context &context, int ft_id, IntVec &features);
  void put(const Context &context, int ft_id, const IntVec &features);
  void raw_get(const Context &context, int ft_id, IntVec &features) const;
  IntVecVecVecVec in2fs; // index in the sequence, length of segment -> feature template ID -> [f]
};

////////////////////////////////////////////////////////////
// Single

struct SingleFeatureCache {
  virtual ~SingleFeatureCache() { }
  virtual IntDouble get(const Context &context, int ft_id) = 0;
  virtual void put(const Context &context, int ft_id, IntDouble fv) = 0;
  virtual IntDouble raw_get(const Context &context, int ft_id) const = 0;
};

struct SinglePtFeatureCache : public SingleFeatureCache {
  IntDouble get(const Context &context, int ft_id);
  void put(const Context &context, int ft_id, IntDouble fv);
  IntDouble raw_get(const Context &context, int ft_id) const;
  IntDoubleVecVec i2fv; // index in the sequence -> feature template ID -> f, v
};

struct SingleSegFeatureCache : public SingleFeatureCache {
  IntDouble get(const Context &context, int ft_id);
  void put(const Context &context, int ft_id, IntDouble fv);
  IntDouble raw_get(const Context &context, int ft_id) const;
  IntDoubleVecVecVec in2fv; // index in the sequence, length of segment -> feature template ID -> f, v
};

////////////////////////////////////////////////////////////
// SingleBool

struct SingleBoolFeatureCache {
  virtual ~SingleBoolFeatureCache() { }
  virtual int get(const Context &context, int ft_id) = 0;
  virtual void put(const Context &context, int ft_id, int f) = 0;
  virtual int raw_get(const Context &context, int ft_id) const = 0;
};

struct SingleBoolPtFeatureCache : public SingleBoolFeatureCache {
  int get(const Context &context, int ft_id);
  void put(const Context &context, int ft_id, int f);
  int raw_get(const Context &context, int ft_id) const;
  IntVecVec i2f; // index in the sequence -> feature template ID -> f
};

struct SingleBoolSegFeatureCache : public SingleBoolFeatureCache {
  int get(const Context &context, int ft_id);
  void put(const Context &context, int ft_id, int f);
  int raw_get(const Context &context, int ft_id) const;
  IntVecVecVec in2f; // index in the sequence, length of segment -> feature template ID -> f
};

#endif
