#include "map-db.h"
#include "basic/stats.h"

void MapDB::read(const char *file, StrDB *phrase_db) {
  track("MapDB::read()", file, true);
  char buf[1024];
  ifstream in(file);
  assert2(in, file);

  this->phrase_db = phrase_db;
  while(in.getline(buf, sizeof(buf))) {
    // Read words
    char *words = strtok(buf, "\t");
    assert(words);
    int p = (*phrase_db)[words];

    assert2(!contains(map, p), "Duplicate entry on line " << len(map)+1);

    // Read feature vectors
    IntDoubleVec feature_values;
    while(true) {
      char *name = strtok(NULL, " ");
      if(!name) break;
      char *val = strtok(NULL, "\t");
      assert(val);
      int f = feature_names[name];
      double v = atof(val);
      assert(v >= -1e10 && v <= 1e10);
      feature_values.push_back(IntDouble(f, v));
    }

    map[p] = feature_values;
  }

  logs("Read " << len(map) << " strengths of " << len(map) << " strings");
}

void MapDB::get_mapping(const char *s, IntDoubleVec &feature_values) const {
  int p = phrase_db->lookup(s, false, -1);
  if(p == -1) return;
  FeatureMap::const_iterator it = map.find(p);
  if(it != map.end()) feature_values = it->second;
}

const char *MapDB::get_name(int f) const {
  return feature_names[f];
}

void MapDB::normalize() {
  track("MapDB::normalize()", "", false);
}
