#ifndef __PARAMS_H__
#define __PARAMS_H__

#include "basic/stl-basic.h"

////////////////////////////////////////////////////////////

class Params {
public:
  Params() { nf = 0; clear_weights(); }
  void resize(int nf);

  void read(const char *file);
  void write(const char *file);

  int num_nonzero_weights() const;
  bool should_keep_feature(int f) const;

  void clear_plain_weights();
  void begin_update() { curr_time++; }
  void end_update() { }

  int set_keep_feature_from_weights();
  void update_weight(int f, double incr);
  double get_weight(bool use_averaged, int f);
  void clear_weights();

  DoublePair checksum() const;

  int size() const { return nf; }

  BoolVec keep_feature; // f -> whether to use feature f.

private:
  int nf; // Number of features.
  DoubleVec weights;
  DoubleVec averaged_weights;

  // The goal is to update averaged_weights efficiently.  averaged_weights is
  // the average (sum) over the weights after each weight update round.  But a
  // weight update round updates only very few of the actual weights.  So,
  // by keeping track of the "time" a weight was updated, we can compute
  // how many times (curr_time-last_updated_time[f]) a feature weight needs
  // to be multiplied.
  // The invariant is that after updating weights, averaged_weights[f]
  // has an additional (curr_time-last_updated_time[f])*weights[f].
  IntVec last_updated_time;
  int curr_time;
};

#endif
