#ifndef __PERFORMANCE_H__
#define __PERFORMANCE_H__

#include <iostream>
#include "token.h"
#include "domain.h"
#include "basic/stats.h"

using namespace std;

class Performance {
public:
  Performance(const Domain &D) : D(D) { }

  double F1() const { return StatFig::F1(precision, recall); }

  void update_stats(const Sequence &seq, const IntVec &truth_tags, const IntVec &pred_tags);
  void update_stats(const Sequence &seq, const SegVec &truth_segs, const SegVec &pred_segs);
  void update_stats_tag(const Sequence &seq, const IntVec &truth_tags, const IntVec &pred_tags);
  void update_stats_seg(const Sequence &seq, const SegVec &truth_segs, const SegVec &pred_segs);

  void print_confusion() const;
  void print_confusion(const StrDB &db, const IntVecVec &confusion, int skip) const;

  void operator=(const Performance &perf);

public:
  const Domain &D;

  StatFig sentence_error, word_error;
  StatFig precision, recall; // segment precision and recall

  IntVec errors; // Indicies of the examples on which we made an error.

  // Truth, Pred -> count
  IntVecVec tag_confusion, seg_confusion;
};

ostream &operator<<(ostream &out, const Performance &perf);

#endif
