#ifndef __SEG_FEATURES_H__
#define __SEG_FEATURES_H__

#include "features.h"
#include "strength-db.h"

struct F_PrevSeg_CurrSeg : public SingleBoolFeatureTemplate {
  F_PrevSeg_CurrSeg(Domain &D);
  int get_feature(const Context &context);
  string description() const;
  string description(int f) const;
  FeatureType type() const { return FTYPE_SEG; }
  bool only_on_input() const { return false; }
};

struct F_CurrSeg : public SingleBoolFeatureTemplate {
  F_CurrSeg(Domain &D);
  int get_feature(const Context &context);
  string description() const;
  string description(int f) const;
  FeatureType type() const { return FTYPE_SEG; }
  bool only_on_input() const { return false; }
};

// map(phrase)
struct F_Phrase : public SingleBoolFeatureTemplate {
  F_Phrase(Domain &D, TransformMap *map);
  int get_feature(const Context &context);
  string description() const;
  string description(int f) const;
  FeatureType type() const { return FTYPE_SEG; }
  bool only_on_input() const { return true; }

  TransformMap *map;
};

// pos_phrase = pos_s ... pos_e
struct F_POSPhrase : public SingleBoolFeatureTemplate {
  F_POSPhrase(Domain &D);
  int get_feature(const Context &context);
  string description() const;
  string description(int f) const;
  FeatureType type() const { return FTYPE_SEG; }
  bool only_on_input() const { return true; }
};

// Apply the feature template to a particular point relative to the segment.
enum Focus { FOCUS_BEGIN, FOCUS_END };
Focus parse_focus(const string &s);

struct F_SingleBoolFocusPhrase : public SingleBoolFeatureTemplate {
  F_SingleBoolFocusPhrase(SingleBoolFeatureTemplate &ft, Focus focus);
  int get_feature(const Context &context);
  string description() const;
  string description(int f) const;
  string raw_description() const;
  FeatureType type() const { return FTYPE_SEG; }
  bool only_on_input() const { return true; }

  SingleBoolFeatureTemplate &ft;
  Focus focus;
};

struct F_FocusPhrase : public MultiFeatureTemplate {
  F_FocusPhrase(MultiFeatureTemplate &ft, Focus focus);
  void get_feature_values(const Context &context, IntDoubleVec &feature_values);
  string description() const;
  string description(int f) const;
  string raw_description() const;
  FeatureType type() const { return FTYPE_SEG; }
  bool only_on_input() const { return true; }

  MultiFeatureTemplate &ft;
  Focus focus;
};

// Apply the feature template to every position on the segment.
struct F_AggregatePhrase : public MultiFeatureTemplate {
  F_AggregatePhrase(MultiFeatureTemplate &ft, int start);
  void get_feature_values(const Context &context, IntDoubleVec &feature_values);

  string description() const;
  string description(int f) const;
  string raw_description() const;
  FeatureType type() const { return FTYPE_SEG; }
  bool only_on_input() const { return true; }

  MultiFeatureTemplate &ft;
  int start;
};

// length(phrase)
struct F_PhraseLength : public SingleBoolFeatureTemplate {
  F_PhraseLength(Domain &D);
  int get_feature(const Context &context);
  string description() const;
  string description(int f) const;
  FeatureType type() const { return FTYPE_SEG; }
  bool only_on_input() const { return true; }
};

// strength(phrase)[idx]
struct F_PhraseStrength : public SingleFeatureTemplate {
  F_PhraseStrength(Domain &D, const StrengthDB &strength_db, double scale, double power, int idx);
  IntDouble get_feature_value(const Context &context);
  string description() const;
  string description(int f) const;
  FeatureType type() const { return FTYPE_SEG; }
  bool only_on_input() const { return true; }

  const StrengthDB &strength_db;
  double scale, power;
  int idx;
};

#endif
