#ifndef __SEG_TAG_H__
#define __SEG_TAG_H__

#include "basic/strdb.h"

////////////////////////////////////////////////////////////

// A segment.
struct Seg {
  Seg() : a(-1), n(0) { }
  Seg(int a, int n) : a(a), n(n) { }
  int a, n; // if a != -1, tag a spans [i, i+n)
};
typedef vector<Seg> SegVec;
typedef vector<SegVec> SegVecVec;

inline bool operator==(const Seg &s1, const Seg &s2) {
  return s1.a == s2.a && s1.n == s2.n;
}

////////////////////////////////////////////////////////////

struct SegTag {
  SegTag() : a(-1), begin(false) { }
  SegTag(int a, bool begin) : a(a), begin(begin) { }
  SegTag(const char *s, StrDB &db) { parse(s, db); }

  int parse(const char *s, StrDB &db);
  string str(const StrDB &db) const;

  int a;
  bool begin;

  static const char *no_seg_str;
};

inline bool operator==(const SegTag &stag1, const SegTag &stag2) {
  return stag1.a == stag2.a && stag1.begin == stag2.begin;
}

inline istream &operator>>(istream &in, SegTag &stag) {
  in >> stag.a >> stag.begin;
  return in;
}

inline ostream &operator<<(ostream &out, const SegTag &stag) {
  out << stag.a << ' ' << stag.begin;
  return out;
}

struct show_SegTag {
  show_SegTag(const SegTag &tag, const StrDB &db) : tag(tag), db(db) { }
  const SegTag &tag;
  const StrDB &db;
};
inline ostream &operator<<(ostream &out, const show_SegTag &show) {
  return out << show.tag.str(show.db);
}

typedef vector<SegTag> SegTagVec;

#endif
