#include "sequence.h"

string Sequence::phrase(const Domain &D, int i, int n) {
  ostringstream out;
  assert(i >= 0 && i+n <= len(tokens));
  for(int j = i; j < i+n; j++) {
    if(j > i) out << ' ';
    out << D.word_db[tokens[j].word];
  }
  return out.str();
}

string Sequence::pos_phrase(const Domain &D, int i, int n) {
  ostringstream out;
  assert(i >= 0 && i+n <= len(tokens));
  for(int j = i; j < i+n; j++) {
    if(j > i) out << ' ';
    out << D.pos_db[tokens[j].pos];
  }
  return out.str();
}

////////////////////////////////////////////////////////////

SegVec Sequence::extract_segs() const {
  SegVec segs(len(tokens));
  foridx(i, len(tokens))
    segs[i] = tokens[i].seg;
  return segs;
}

IntVec Sequence::extract_tags() const {
  IntVec tags(len(tokens));
  foridx(i, len(tokens))
    tags[i] = tokens[i].tag;
  return tags;
}

SegVec Sequence::tags_to_segs(const Domain &D, const IntVec &tags) {
  int start_a = -1;
  int start_i = -1;
  int n = len(tags);

  SegVec segs(n);
  foridx(i, n) segs[i].a = -1;

  foridx(i, n) {
    SegTag stag = D.stags[tags[i]];
    // Old tag ends.
    if(start_i != -1 && (stag.begin || stag.a != start_a)) {
      segs[start_i] = Seg(start_a, i-start_i);
      start_a = -1;
    }
    // New tag begins.
    if(stag.begin) {
      assert(start_a == -1);
      start_a = stag.a;
      start_i = i;
    }
  }
  if(start_i != -1)
    segs[start_i] = Seg(start_a, n-start_i);
  return segs;
}

IntVec Sequence::segs_to_tags(const Domain &D, const SegVec &segs) {
  int n = len(segs);
  IntVec tags(n, D.no_seg_tag);

  foridx(i, n) {
    SegTag stag;
    if(segs[i].a == -1) continue;
    assert(i+segs[i].n <= n);
    foridx(j, segs[i].n)
      tags[i+j] = D.stag_to_tag(SegTag(segs[i].a, j == 0));
  }
  return tags;
}

void Sequence::set_segs(const Domain &D) {
  SegVec segs = tags_to_segs(D, extract_tags());
  forvec(i, Seg, seg, segs)
    tokens[i].seg = seg;
}

////////////////////////////////////////////////////////////

ostream &operator<<(ostream &out, const show_Sequence &show) {
  forvec(i, const Token &, token, show.seq) {
    if(i > 0) out << ' ';
    out << show.D.word_db[token.word] << '(' << show.D.tag_db[token.tag];
    if(show.tags[i] != token.tag)
      out << '/' << show.D.tag_db[show.tags[i]];
    out << ')';
  }
  return out;
}
