#ifndef __SEQUENCE_H__
#define __SEQUENCE_H__

#include "token.h"

struct Sequence {
  IntVec extract_tags() const;
  SegVec extract_segs() const;
  static IntVec segs_to_tags(const Domain &D, const SegVec &segs);
  static SegVec tags_to_segs(const Domain &D, const IntVec &tags);

  const Token &operator[](int i) const { return tokens[i]; }
        Token &operator[](int i)       { return tokens[i]; }

  void clear() { tokens.clear(); }
  void set_segs(const Domain &D);
  Token &push_back(const Token &token = Token()) { tokens.push_back(token); return tokens[len(tokens)-1]; }

  // Extract segments
  string phrase(const Domain &D, int i, int n);
  string pos_phrase(const Domain &D, int i, int n);

  void pop_back() { tokens.pop_back(); }
  int size() const { return len(tokens); }

  // Caches
#define define_cache(t, T) \
  T##FeatureCache &t##_cache(FeatureType type) { \
    if(type == FTYPE_PT)  return t##_pt_cache; \
    if(type == FTYPE_SEG) return t##_seg_cache; \
    assert(0); \
  }
  define_cache(multi, Multi);
  define_cache(multi_bool, MultiBool);
  define_cache(single, Single);
  define_cache(single_bool, SingleBool);
#undef define_cache

  MultiPtFeatureCache multi_pt_cache;
  MultiSegFeatureCache multi_seg_cache;
  MultiBoolPtFeatureCache multi_bool_pt_cache;
  MultiBoolSegFeatureCache multi_bool_seg_cache;
  SinglePtFeatureCache single_pt_cache;
  SingleSegFeatureCache single_seg_cache;
  SingleBoolPtFeatureCache single_bool_pt_cache;
  SingleBoolSegFeatureCache single_bool_seg_cache;

private:
  vector<Token> tokens;
};

typedef vector<Sequence> SequenceVec;

struct show_Sequence {
  show_Sequence(const Domain &D, const Sequence &seq, const IntVec &tags) : D(D), seq(seq), tags(tags) { }
  const Domain &D;
  const Sequence &seq;
  const IntVec &tags;
};
ostream &operator<<(ostream &out, const show_Sequence &show);

#endif
