#include "basic/str.h"
#include "str-transform.h"

////////////////////////////////////////////////////////////

string T_Prefix::transform(const char *s) const { return substr(s, 0, k); }
string T_Prefix::description() const { return str_printf("prefix%d", k); }

string T_Suffix::transform(const char *s) const { return substr(s, -k); }
string T_Suffix::description() const { return str_printf("suffix%d", k); }

string T_Lowercase::transform(const char *s) const { return tolower(s); }
string T_Lowercase::description() const { return "lowercase"; }

////////////////////////////////////////////////////////////

T_Contains::T_Contains(string descrip) : descrip(descrip) {
  hit.resize(256);
  if(descrip == "DIGIT") {
    for(char c = '0'; c <= '9'; c++) hit[c] = true;
  }
  else if(descrip == "UPPER") {
    for(char c = 'A'; c <= 'Z'; c++) hit[c] = true;
  }
  else if(len(descrip) == 1)
    hit[descrip[0]] = true;
  else
    assert2(false, "T_Contains: Invalid description: " << descrip);
}

string T_Contains::transform(const char *s) const {
  while(*s) if(hit[*s++]) return "Y";
  return "N";
}

string T_Contains::description() const {
  return str_printf("contains(%s)", descrip.c_str());
}

////////////////////////////////////////////////////////////

char get_form(char c) {
  if(isalpha(c)) return isupper(c) ? 'A' : 'a';
  if(isdigit(c)) return '0';
  return c;
}

string T_Form::transform(const char *s) const {
  string t;
  char last_form = '\0';
  int n = strlen(s);
  foridx(i, n) {
    char form = get_form(s[i]);
    if(!brief || form != last_form) t += form;
    last_form = form;
  }
  return t;
}

string T_Form::description() const {
  return str_printf("%corm", "Ff"[brief]);
}

////////////////////////////////////////////////////////////

int TransformMap::operator[](int s) {
  int t = expand_get(s2t, s, -1);
  if(t == -1) {
    t = t_db[transformer->transform(s_db[s]).c_str()];
    s2t[s] = t;
  }
  return t;
}

////////////////////////////////////////////////////////////

string T_Crypt::transform(const char *s) const {
  str_hf hf;
  return str_printf("%d", (hf(s) * (key+35)) % range);
}

string T_Crypt::description() const {
  return str_printf("hash%d-%d", key, range);
}

