#include "strength-db.h"
#include "basic/stats.h"

void StrengthDB::read(const char *file, StrDB &word_db) {
  track("StrengthDB::read()", file, true);
  char buf[1024];
  ifstream in(file);
  assert2(in, file);

  nstrengths = -1;

  while(in.getline(buf, sizeof(buf))) {
    // Read words
    char *words = strtok(buf, "\t");

    // Read strengths
    DoubleVec strengths;
    for(char *strength_str = strtok(NULL, " "); strength_str; strength_str = strtok(NULL, " ")) {
      double strength = atof(strength_str);
      strengths.push_back(strength);
    }
    assert(words && len(strengths) > 0);

    assert2(nstrengths == -1 || nstrengths == len(strengths), "Number of strengths varies: " << nstrengths << ' ' << len(strengths));
    nstrengths = len(strengths);

    // Convert word
    IntVec ws;
    for(char *w_str = strtok(words, " "); w_str; w_str = strtok(NULL, " ")) {
      int w = word_db[w_str];
      ws.push_back(w);
    }

    assert2(!contains(strengths_map, ws), "Duplicate entry on line " << len(strengths_map)+1);
    strengths_map[ws] = strengths;
  }

  logs("Read " << nstrengths << " strengths of " << len(strengths_map) << " strings");
}

void StrengthDB::normalize() {
  track("StrengthDB::normalize()", "", false);

  track_foridx(i, nstrengths, "Strength", false) {
    FullStatFig fig; // Not very efficient
    formap(const IntVec &, ws, DoubleVec &, strengths, IntVecDoubleVecMap, strengths_map)
      fig.add(strengths[i]); 

    double mean = fig.mean(), stddev = fig.stddev();
    logs("mean = " << mean << ", stddev = " << stddev);
    formap(const IntVec &, ws, DoubleVec &, strengths, IntVecDoubleVecMap, strengths_map)
      strengths[i] = (strengths[i] - mean) / stddev;
  }
}

double StrengthDB::get_strength(int w1, int w2, int i) const {
  IntVec ws;
  ws.push_back(w1);
  ws.push_back(w2);
  return get_strength(ws, i);
}

double StrengthDB::get_strength(const IntVec &ws, int i) const {
  IntVecDoubleVecMap::const_iterator it = strengths_map.find(ws);
  if(it == strengths_map.end()) return 0;
  const DoubleVec &strengths = it->second;
  assert(i >= 0 && i < len(strengths));
  return strengths[i];
}
