#ifndef __TAG_FEATURES_H__
#define __TAG_FEATURES_H__

#include "features.h"
#include "strength-db.h"
#include "map-db.h"

// prev_tag, curr_tag
struct F_Prev2Tag_PrevTag_CurrTag : public SingleBoolFeatureTemplate {
  F_Prev2Tag_PrevTag_CurrTag(Domain &D);
  int get_feature(const Context &context);
  string description() const;
  string description(int f) const;
  FeatureType type() const { return FTYPE_PT; }
  bool only_on_input() const { return false; }
};

// prev_tag, curr_tag
struct F_PrevTag_CurrTag : public SingleBoolFeatureTemplate {
  F_PrevTag_CurrTag(Domain &D);
  int get_feature(const Context &context);
  string description() const;
  string description(int f) const;
  FeatureType type() const { return FTYPE_PT; }
  bool only_on_input() const { return false; }
};

// curr_tag
struct F_CurrTag : public SingleBoolFeatureTemplate {
  F_CurrTag(Domain &D);
  int get_feature(const Context &context);
  string description() const;
  string description(int f) const;
  FeatureType type() const { return FTYPE_PT; }
  bool only_on_input() const { return false; }
};

// map(word(i+offset))
struct F_Word : public SingleBoolFeatureTemplate {
  F_Word(Domain &D, int offset, TransformMap *map);
  int get_feature(const Context &context);
  string description() const;
  string description(int f) const;
  FeatureType type() const { return FTYPE_PT; }
  bool only_on_input() const { return true; }

  int offset;
  TransformMap *map;
};

// map(words(i+offset..i+offset+n-1))
struct F_Words : public SingleBoolFeatureTemplate {
  F_Words(Domain &D, int offset, int n, TransformMap *map);
  int get_feature(const Context &context);
  string description() const;
  string description(int f) const;
  FeatureType type() const { return FTYPE_PT; }
  bool only_on_input() const { return true; }

  int offset, n;
  TransformMap *map;
};

// curr_tag, pos[i+offset]
struct F_POS : public SingleBoolFeatureTemplate {
  F_POS(Domain &D, int offset);
  int get_feature(const Context &context);
  string description() const;
  string description(int f) const;
  FeatureType type() const { return FTYPE_PT; }
  bool only_on_input() const { return true; }

  int offset;
};

// strength(words(i-1..i))[idx]
struct F_Strength : public SingleFeatureTemplate {
  F_Strength(Domain &D, const StrengthDB &strength_db, double scale, double power, int idx);
  IntDouble get_feature_value(const Context &context);
  string description() const;
  string description(int f) const;
  FeatureType type() const { return FTYPE_PT; }
  bool only_on_input() const { return true; }

  const StrengthDB &strength_db;
  double scale, power;
  int idx;
};

// Most general type of feature mapping a vector of words to a set of (string, value) pairs
// (words(i+offset..i+offset+n-1))
struct F_Map : public MultiFeatureTemplate {
  F_Map(Domain &D, int offset, int n, const MapDB &map_db);
  void get_feature_values(const Context &context, IntDoubleVec &feature_values);
  string description() const;
  string description(int f) const;
  FeatureType type() const { return FTYPE_PT; }
  bool only_on_input() const { return true; }

  int offset, n;
  const MapDB &map_db;
};

#endif
